import sys

from logging import Handler, LogRecord
from concurrent.futures import ThreadPoolExecutor

from kilmlogger.services.scribe.client import ScribeBaseClient, DefaultScribeClient
from kilmlogger.constants import ACCEPTED_LEVELS


executors = ThreadPoolExecutor(max_workers=50)


class GRPCEventStreamingHandler(Handler):
    def __init__(self, client: ScribeBaseClient = DefaultScribeClient(), accepted_levels: list[str] = ACCEPTED_LEVELS) -> None:
        """
        Initialize the handler.

        If stream is not specified, sys.stderr is used.
        """
        Handler.__init__(self)
        self._client = client
        self.accepted_levels = accepted_levels

    def emit(self, record: LogRecord) -> None:
        # NOTE: Only sent log to scribe and DP if the log level is in ACCEPTED_LEVELS
        if record.levelname not in self.accepted_levels:
            return

        try:
            executors.submit(self._client.log, msg=record.msg)
        except Exception:
            sys.stdout.write("--- Logging error ---\n")



import logging

import structlog

from ujson import dumps
from structlog.dev import (
    Column,
    KeyValueColumnFormatter,
    BRIGHT,
    DIM,
    CYAN,
    RESET_ALL,
    RED,
    YELLOW,
    GREEN,
    RED_BACK,
)

from kilmlogger.constants import (
    ACCEPTED_LEVELS,
    DEFAULT_FILE_LOG,
    REPLACE_KEY_FOR_TIMESTAMP,
    REPLACE_KEY_FOR_EVENT,
    TIME_FIELD_NAME,
    MESSAGE_FIELD_NAME,
    LEVEL_FIELD_NAME,
    CORRELATION_FIELD_NAME,
    EXTRA_DATA_FIELD_NAME,
    METRICS_FIELD_NAME,
)
from kilmlogger.utils import add_correlation
from kilmlogger.styles import LogLevelColumnFormatter
from kilmlogger.envs import envs


LOG_LEVEL_COLORS = {
    "critical": RED,
    "exception": RED,
    "error": RED,
    "warn": YELLOW,
    "warning": YELLOW,
    "info": GREEN,
    "debug": GREEN,
    "notset": RED_BACK,
}


class KilmLoggerConfiguration:
    def __init__(
        self,
        logging_filename: str = DEFAULT_FILE_LOG,
        logging_level: int = logging.INFO,
        accepted_levels: list[str] = ACCEPTED_LEVELS,
        console_logging_level: int = logging.INFO,
        use_async: bool = False,
    ) -> None:
        self.logging_filename = logging_filename
        self.logging_level = logging_level
        self.accepted_levels = accepted_levels
        self.console_logging_level = console_logging_level
        self.formatters = self._use_default_formatters()
        self.handlers = self._use_default_handlers()
        self.loggers = self._use_default_loggers()
        self.use_async = use_async

    def use_default_configuration(self) -> None:
        logging.config.dictConfig(
            {
                "version": 1,
                "disable_existing_loggers": False,
                "formatters": self.formatters,
                "handlers": self.handlers,
                "loggers": self.loggers,
            }
        )

        self._config_structlog()

    def _config_structlog(self) -> None:
        structlog.configure(
            processors=[
                structlog.stdlib.filter_by_level,
                structlog.stdlib.add_log_level,
                add_correlation,
                structlog.processors.TimeStamper(
                    fmt=envs.TIME_FORMAT, key=REPLACE_KEY_FOR_TIMESTAMP
                ),
                structlog.processors.StackInfoRenderer(),
                structlog.processors.format_exc_info,
                structlog.processors.EventRenamer(to=REPLACE_KEY_FOR_EVENT),
                structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
            ],
            wrapper_class=structlog.stdlib.AsyncBoundLogger if self.use_async else structlog.stdlib.BoundLogger,
            logger_factory=structlog.stdlib.LoggerFactory(),
            cache_logger_on_first_use=True,
        )

    def _use_default_formatters(self) -> dict:
        return {
            "colored": {
                "()": structlog.stdlib.ProcessorFormatter,
                "processors": [
                    structlog.stdlib.ProcessorFormatter.remove_processors_meta,
                    structlog.dev.ConsoleRenderer(
                        colors=True,
                        columns=[
                            Column(
                                "",
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=DIM,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                ),
                            ),
                            Column(
                                TIME_FIELD_NAME,
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=RESET_ALL,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                ),
                            ),
                            Column(
                                LEVEL_FIELD_NAME,
                                LogLevelColumnFormatter(
                                    LOG_LEVEL_COLORS, reset_style=RESET_ALL
                                ),
                            ),
                            Column(
                                CORRELATION_FIELD_NAME,
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=CYAN,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                    width=30,
                                ),
                            ),
                            Column(
                                MESSAGE_FIELD_NAME,
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=BRIGHT,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                ),
                            ),
                            Column(
                                EXTRA_DATA_FIELD_NAME,
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=BRIGHT,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                ),
                            ),
                            Column(
                                METRICS_FIELD_NAME,
                                KeyValueColumnFormatter(
                                    key_style=None,
                                    value_style=BRIGHT,
                                    reset_style=RESET_ALL,
                                    value_repr=str,
                                ),
                            ),
                        ],
                    ),
                ],
            },
            "json": {
                "()": structlog.stdlib.ProcessorFormatter,
                "processor": structlog.processors.JSONRenderer(serializer=dumps, ensure_ascii=False),
                "foreign_pre_chain": [
                    structlog.stdlib.PositionalArgumentsFormatter(),
                    structlog.stdlib.ProcessorFormatter.wrap_for_formatter,
                ],
            },
        }

    def _use_default_handlers(self) -> dict:
        return {
            "default": {
                "level": self.console_logging_level,
                "class": "logging.StreamHandler",
                "formatter": "colored",
            },
            "file": {
                "level": logging.DEBUG,
                "class": "logging.handlers.WatchedFileHandler",
                "filename": self.logging_filename,
                "formatter": "json",
            },
            "scribe": {
                "level": self.logging_level,
                "class": "kilmlogger.handler.GRPCEventStreamingHandler",
                "accepted_levels": self.accepted_levels,
            },
        }

    def _use_default_loggers(self) -> dict:
        return {
            "kilmlogger": {
                "handlers": ["default", "file", "scribe"],
                "level": logging.DEBUG,
                "propagate": True,
            },
        }



from pydantic_settings import BaseSettings


class Env(BaseSettings):
    # The name of the environment.
    ENVIRONMENT: str | None = None
    SCRIBE_TIMEOUT: float = 0.05

    SCRIBE_HOST: str | None = None
    SCRIBE_PORT: str | None = None

    # The name of the application.
    APP_NAME: str | None = None

    # The version of the application.
    APP_VERSION: str | None = None

    # The category of the logging.
    DP_CATEGORY: str | None = None
    DP_LOG: str | None = None
    
    # Time format for the log.
    TIME_FORMAT: str = "%Y-%m-%d %H:%M:%S.%f %Z"


envs = Env()



import json
import grpc

from datetime import datetime

from kilmlogger.services.scribe.grpc.scribelog_pb2_grpc import ScribeLogServiceStub
from kilmlogger.services.scribe.grpc.scribelog_pb2 import (
    ListOfEntryRequest,
    LogEntryRequest,
    LogEntry,
)

from kilmlogger.envs import envs


class ScribeBaseClient(object):
    def __init__(
        self,
        host: str | None = envs.SCRIBE_HOST,
        port: str | None = envs.SCRIBE_PORT,
        category: str | None = envs.DP_CATEGORY,
        app_name: str | None = envs.APP_NAME,
        app_prop: str | None = envs.ENVIRONMENT,
        dp_cate: str | None = envs.DP_CATEGORY,
        dp_log: str | None = envs.DP_LOG,
    ):
        self.host = host
        self.port = port
        self.category = category
        self.app_name = app_name
        self.app_prop = app_prop
        self.dp_cate = dp_cate
        self.dp_log = dp_log

    @staticmethod
    def build_grpc_stub(host: str, port: str) -> ScribeLogServiceStub:
        channel: grpc.Channel = grpc.insecure_channel(
            target=f"{host}:{port}",
            compression=grpc.Compression.Gzip,
        )
        return ScribeLogServiceStub(channel)

    @property
    def grpc_stub(self) -> ScribeLogServiceStub:
        if not hasattr(self, "_grpc_stub"):
            self._grpc_stub = self.build_grpc_stub(self.host, self.port)
        return self._grpc_stub

    def log(self, msg: dict) -> None:
        if msg.get("metric_only"):
            self.grpc_stub.sendMultiLog(
                self._build_request(
                    category=msg.get("category"),
                    json_param=self._build_dp_params(msg),
                    start_time=msg.get("start_time"),
                    command=msg.get("command"),
                    sub_command=msg.get("sub_command"),
                    result=msg.get("result"),
                ),
                timeout=envs.SCRIBE_TIMEOUT,
            )
        else:
            self.grpc_stub.sendMultiLogV2(
                self._build_request(
                    category=msg.get("category"),
                    json_param=self._build_dp_params(msg),
                    start_time=msg.get("start_time"),
                    dp_log=msg.get("dp_log"),
                    dp_cate=msg.get("dp_cate"),
                    command=msg.get("command"),
                    sub_command=msg.get("sub_command"),
                    result=msg.get("result"),
                ),
                timeout=envs.SCRIBE_TIMEOUT,
            )


class DefaultScribeClient(ScribeBaseClient):
    def _build_request(
        self,
        category: str | None = None,
        json_param: dict = {},
        start_time: int | None = None,
        dp_log: str | None = None,
        dp_cate: str | None = None,
        command: int | None = None,
        sub_command: int | None = None,
        result: int | None = None,
    ) -> ListOfEntryRequest:
        log_time = int(datetime.utcnow().timestamp() * 1000)
        execute_time: int | None = log_time - start_time if start_time else None
        return ListOfEntryRequest(
            logEntryRequest=[
                LogEntryRequest(
                    category=category or self.category,
                    app_name=self.app_name,
                    app_prop=self.app_prop,
                    timestamp=log_time,
                    log_entry=LogEntry(
                        json_param=json.dumps(json_param),
                        start_time=start_time,
                        dp_log=dp_log or self.dp_log,
                        dp_cate=dp_cate or self.dp_cate,
                        command=command,
                        sub_command=sub_command,
                        result=result,
                        execute_time=execute_time,
                    ),
                )
            ]
        )

    def _build_dp_params(self, msg: dict) -> dict:
        return {
            "log_time": msg["time"],
            "app_name": self.app_name,
            "app_mode": self.app_prop,
            "event_category": msg["level"],
            "correlation_id": msg["correlation_id"],
            "message": {
                "time": msg["time"],
                "content": msg["msg"],
            },
            "metrics": msg.get("metrics", {}),
            "extra_data": msg.get("extra_data", {}),
        }



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: scribelog.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x0fscribelog.proto\x12\x1e\x63om.vng.zlb.scribelog.grpc.lib"\x15\n\x04Ping\x12\r\n\x05\x63ount\x18\x01 \x01(\x03"\x15\n\x04Pong\x12\r\n\x05\x63ount\x18\x01 \x01(\x03"^\n\x12ListOfEntryRequest\x12H\n\x0flogEntryRequest\x18\x01 \x03(\x0b\x32/.com.vng.zlb.scribelog.grpc.lib.LogEntryRequest"d\n\x16ListOfActionLogRequest\x12J\n\x10\x61\x63tionLogRequest\x18\x01 \x03(\x0b\x32\x30.com.vng.zlb.scribelog.grpc.lib.ActionLogRequest"9\n\x10\x41\x63tionLogRequest\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x12\n\njson_param\x18\x02 \x01(\t"\x97\x01\n\x0fLogEntryRequest\x12\x11\n\ttimestamp\x18\x01 \x01(\x03\x12\x10\n\x08\x63\x61tegory\x18\x02 \x01(\t\x12\x10\n\x08\x61pp_name\x18\x03 \x01(\t\x12\x10\n\x08\x61pp_prop\x18\x04 \x01(\t\x12;\n\tlog_entry\x18\x05 \x01(\x0b\x32(.com.vng.zlb.scribelog.grpc.lib.LogEntry"6\n\x0eLogEntryResult\x12\r\n\x05\x65rror\x18\x01 \x01(\x05\x12\x15\n\rerror_message\x18\x02 \x01(\t"\x9e\x02\n\x08LogEntry\x12\n\n\x02ip\x18\x01 \x01(\t\x12\x0e\n\x06src_id\x18\x02 \x01(\x05\x12\x0e\n\x06\x64\x65s_id\x18\x03 \x01(\x05\x12\x0f\n\x07\x63ommand\x18\x04 \x01(\r\x12\x13\n\x0bsub_command\x18\x05 \x01(\r\x12\x0e\n\x06result\x18\x06 \x01(\x05\x12\x12\n\nstart_time\x18\x07 \x01(\x03\x12\x14\n\x0c\x65xecute_time\x18\x08 \x01(\x03\x12\x12\n\njson_param\x18\t \x01(\t\x12\x16\n\x0ejson_ext_param\x18\n \x01(\t\x12\x13\n\x0b\x63lient_type\x18\x0b \x01(\r\x12\x16\n\x0e\x63lient_version\x18\x0c \x01(\x05\x12\x0c\n\x04type\x18\r \x01(\r\x12\x0e\n\x06\x64p_log\x18\x0e \x01(\t\x12\x0f\n\x07\x64p_cate\x18\x0f \x01(\t2\xbd\x05\n\x10ScribeLogService\x12l\n\x07sendLog\x12/.com.vng.zlb.scribelog.grpc.lib.LogEntryRequest\x1a..com.vng.zlb.scribelog.grpc.lib.LogEntryResult"\x00\x12t\n\x0csendMultiLog\x12\x32.com.vng.zlb.scribelog.grpc.lib.ListOfEntryRequest\x1a..com.vng.zlb.scribelog.grpc.lib.LogEntryResult"\x00\x12s\n\rsendActionLog\x12\x30.com.vng.zlb.scribelog.grpc.lib.ActionLogRequest\x1a..com.vng.zlb.scribelog.grpc.lib.LogEntryResult"\x00\x12~\n\x12sendMultiActionLog\x12\x36.com.vng.zlb.scribelog.grpc.lib.ListOfActionLogRequest\x1a..com.vng.zlb.scribelog.grpc.lib.LogEntryResult"\x00\x12v\n\x0esendMultiLogV2\x12\x32.com.vng.zlb.scribelog.grpc.lib.ListOfEntryRequest\x1a..com.vng.zlb.scribelog.grpc.lib.LogEntryResult"\x00\x12X\n\x08pingPong\x12$.com.vng.zlb.scribelog.grpc.lib.Ping\x1a$.com.vng.zlb.scribelog.grpc.lib.Pong"\x00\x42\x32\n\x1e\x63om.vng.zlb.scribelog.grpc.libB\x0eScribeLogProtoP\x01\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "scribelog_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = (
        b"\n\036com.vng.zlb.scribelog.grpc.libB\016ScribeLogProtoP\001"
    )
    _globals["_PING"]._serialized_start = 51
    _globals["_PING"]._serialized_end = 72
    _globals["_PONG"]._serialized_start = 74
    _globals["_PONG"]._serialized_end = 95
    _globals["_LISTOFENTRYREQUEST"]._serialized_start = 97
    _globals["_LISTOFENTRYREQUEST"]._serialized_end = 191
    _globals["_LISTOFACTIONLOGREQUEST"]._serialized_start = 193
    _globals["_LISTOFACTIONLOGREQUEST"]._serialized_end = 293
    _globals["_ACTIONLOGREQUEST"]._serialized_start = 295
    _globals["_ACTIONLOGREQUEST"]._serialized_end = 352
    _globals["_LOGENTRYREQUEST"]._serialized_start = 355
    _globals["_LOGENTRYREQUEST"]._serialized_end = 506
    _globals["_LOGENTRYRESULT"]._serialized_start = 508
    _globals["_LOGENTRYRESULT"]._serialized_end = 562
    _globals["_LOGENTRY"]._serialized_start = 565
    _globals["_LOGENTRY"]._serialized_end = 851
    _globals["_SCRIBELOGSERVICE"]._serialized_start = 854
    _globals["_SCRIBELOGSERVICE"]._serialized_end = 1555
# @@protoc_insertion_point(module_scope)
